@@ -10,15 +10,16 @@
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
 #
-# import os
-# import sys
+import os
+import sys
 # sys.path.insert(0, os.path.abspath('.'))
+sys.path.insert(0, os.path.join(os.path.abspath('..'), 'tscode'))
 
 
 # -- Project information -----------------------------------------------------
 
 project = 'TSCoDe'
-copyright = '2021, Nicolò Tampellini'
+copyright = '2021-2023, Nicolò Tampellini'
 author = 'Nicolò Tampellini'
 
 # The full version, including alpha/beta/rc tags
@@ -26,7 +27,7 @@ author = 'Nicolò Tampellini'
 # os.chdir(os.path.dirname(os.path.realpath(__file__)))
 # os.chdir(os.path.dirname(os.getcwd()))
 # from tscode.__main__ import __version__
-release = '0.1.0'
+release = '0.4.0'
 # release='latest'
 
 # -- General configuration ---------------------------------------------------
@@ -34,7 +35,7 @@ release = '0.1.0'
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
-extensions = []
+extensions = ["sphinx.ext.todo", "sphinx.ext.viewcode", "sphinx.ext.autodoc"]
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -55,4 +56,4 @@ html_theme = 'sphinx_rtd_theme'
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['_static']
\ No newline at end of file
+# html_static_path = ['_static']
\ No newline at end of file
