@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,10 +38,11 @@ from sella import Sella
 
 from tscode.algebra import norm, norm_of
 from tscode.graph_manipulations import findPaths, graphize, neighbors
+from tscode.hypermolecule_class import align_structures
 from tscode.settings import COMMANDS, MEM_GB
 from tscode.solvents import get_solvent_line
 from tscode.utils import (HiddenPrints, clean_directory,
-                          get_double_bonds_indexes, molecule_check,
+                          get_double_bonds_indices, molecule_check,
                           scramble_check, time_to_string, write_xyz)
 
 
@@ -124,8 +125,8 @@ def get_ase_calc(embedder):
     Attach the correct ASE calculator
     to the ASE Atoms object.
     embedder: either a TSCoDe embedder object or
-        a 4-element strings tuple containing
-        (calculator, method, procs, solvent)
+    a 4-element strings tuple containing
+    (calculator, method, procs, solvent)
     '''
     if isinstance(embedder, tuple):
         calculator, method, procs, solvent = embedder
@@ -133,7 +134,7 @@ def get_ase_calc(embedder):
     else:
         calculator = embedder.options.calculator
         method = embedder.options.theory_level
-        procs = embedder.options.procs
+        procs = embedder.procs
         solvent = embedder.options.solvent
 
     if calculator == 'XTB':
@@ -178,7 +179,7 @@ def get_ase_calc(embedder):
             orcablocks += get_solvent_line(solvent, calculator, method)
 
         return ORCA(label='temp',
-                    command=f'{command} temp.inp > temp.out 2>&1',
+                    command=f'{command} temp.inp "--oversubscribe" > temp.out 2>&1',
                     orcasimpleinput=method,
                     orcablocks=orcablocks)
 
@@ -212,12 +213,12 @@ def get_ase_calc(embedder):
 
             return calc
 
-def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title=0, traj=None):
+def ase_adjust_spacings(embedder, structure, atomnos, constrained_indices, title=0, traj=None):
     '''
     embedder: TSCoDe embedder object
     structure: TS candidate coordinates to be adjusted
     atomnos: 1-d array with element numbering for the TS
-    constrained_indexes: (n,2)-shaped array of indexes to be distance constrained
+    constrained_indices: (n,2)-shaped array of indices to be distance constrained
     mols_graphs: list of NetworkX graphs, ordered as the single molecules in the TS
     title: number to be used for referring to this structure in the embedder log
     traj: if set to a string, traj+'.traj' is used as a filename for the refinement trajectory.
@@ -226,7 +227,7 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
 
     atoms.calc = get_ase_calc(embedder)
     
-    springs = [Spring(indexes[0], indexes[1], dist) for indexes, dist in embedder.target_distances.items()]
+    springs = [Spring(indices[0], indices[1], dist) for indices, dist in embedder.target_distances.items()]
     # adding springs to adjust the pairings for which we have target distances
 
     # if there are no springs, it is faster (and equivalent) to just do a classical full opitimization
@@ -238,17 +239,17 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
                         embedder.options.calculator,
                         method=embedder.options.theory_level,
                         mols_graphs=embedder.graphs if embedder.embed != 'monomolecular' else None,
-                        procs=embedder.options.procs,
+                        procs=embedder.procs,
                         solvent=embedder.options.solvent,
                         max_newbonds=embedder.options.max_newbonds,
-                        check=(embedder.embed != 'run'),
+                        check=(embedder.embed != 'refine'),
 
                         logfunction=lambda s: embedder.log(s, p=False),
                         title=f'Candidate_{title}'
                     )
 
-    nci_indexes = [indexes for letter, indexes in embedder.pairings_table.items() if letter in ('x', 'y', 'z')]
-    halfsprings = [HalfSpring(i1, i2, 2.5) for i1, i2 in nci_indexes]
+    nci_indices = [indices for letter, indices in embedder.pairings_table.items() if letter in ('x', 'y', 'z')]
+    halfsprings = [HalfSpring(i1, i2, 2.5) for i1, i2 in nci_indices]
     # HalfSprings get atoms involved in NCIs together if they are more than 2.5A apart,
     # but lets them achieve their natural equilibrium distance when closer
 
@@ -272,6 +273,7 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
             atoms.set_constraint(springs)
             # Tightening Springs to improve
             # spacings accuracy, removing PSC
+            # spacings accuracy, removing PSC
 
             opt.run(fmax=0.05, steps=200)
             # final accurate refinement
@@ -281,7 +283,14 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
 
         new_structure = atoms.get_positions()
 
-        success = scramble_check(new_structure, atomnos, constrained_indexes, embedder.graphs)
+        success = scramble_check(new_structure, atomnos, constrained_indices, embedder.graphs)
+        if iterations == 200:
+            exit_str = 'MAX ITER'            
+        elif success:
+            exit_str = 'REFINED'
+        else:
+            exit_str = 'SCRAMBLED'
+        
         if iterations == 200:
             exit_str = 'MAX ITER'            
         elif success:
@@ -293,6 +302,7 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
         exit_str = 'CRASHED'
 
     embedder.log(f'    - {title} {exit_str} ({iterations} iterations, {time_to_string(time.perf_counter()-t_start_opt)})', p=False)
+    embedder.log(f'    - {title} {exit_str} ({iterations} iterations, {time_to_string(time.perf_counter()-t_start_opt)})', p=False)
 
     if exit_str == 'CRASHED':
         return None, None, False
@@ -301,7 +311,7 @@ def ase_adjust_spacings(embedder, structure, atomnos, constrained_indexes, title
 
     return new_structure, energy, success
 
-def ase_saddle(embedder, coords, atomnos, constrained_indexes=None, mols_graphs=None, title='temp', logfile=None, traj=None, freq=False, maxiterations=200):
+def ase_saddle(embedder, coords, atomnos, constrained_indices=None, mols_graphs=None, title='temp', logfile=None, traj=None, freq=False, maxiterations=200):
     '''
     Runs a first order saddle optimization through the ASE package
     '''
@@ -312,9 +322,9 @@ def ase_saddle(embedder, coords, atomnos, constrained_indexes=None, mols_graphs=
     t_start = time.perf_counter()
     with HiddenPrints():
         with Sella(atoms,
-                   logfile=None,
-                   order=1,
-                   trajectory=traj) as opt:
+                    logfile=logfile,
+                    order=1,
+                    trajectory=traj) as opt:
 
             opt.run(fmax=0.05, steps=maxiterations)
             iterations = opt.nsteps
@@ -329,7 +339,7 @@ def ase_saddle(embedder, coords, atomnos, constrained_indexes=None, mols_graphs=
     energy = atoms.get_total_energy() * 23.06054194532933 #eV to kcal/mol
 
     if mols_graphs is not None:
-        success = scramble_check(new_structure, atomnos, constrained_indexes, mols_graphs, max_newbonds=embedder.options.max_newbonds)
+        success = scramble_check(new_structure, atomnos, constrained_indices, mols_graphs, max_newbonds=embedder.options.max_newbonds)
     else:
         success = molecule_check(coords, new_structure, atomnos, max_newbonds=embedder.options.max_newbonds)
 
@@ -337,6 +347,7 @@ def ase_saddle(embedder, coords, atomnos, constrained_indexes=None, mols_graphs=
 
 def ase_vib(embedder, coords, atomnos, logfunction=None, title='temp'):
     '''
+    Calculate frequencies through ASE - returns frequencies and number of negatives (not in use)
     '''
     atoms = Atoms(atomnos, positions=coords)
     atoms.calc = get_ase_calc(embedder)
@@ -368,7 +379,7 @@ def ase_vib(embedder, coords, atomnos, logfunction=None, title='temp'):
 
     return freqs, np.count_nonzero(freqs.imag > 1e-3)
 
-def ase_neb(embedder, reagents, products, atomnos, n_images=6, title='temp', optimizer=LBFGS, logfunction=None, write_plot=False, verbose_print=False):
+def ase_neb(embedder, reagents, products, atomnos, ts_guess=None, n_images=6, mep_override=None, title='temp', optimizer=LBFGS, logfunction=None, write_plot=False, verbose_print=False):
     '''
     embedder: tscode embedder object
     reagents: coordinates for the atom arrangement to be used as reagents
@@ -382,20 +393,42 @@ def ase_neb(embedder, reagents, products, atomnos, n_images=6, title='temp', opt
     return: 3- element tuple with coodinates of highest point along the MEP, its
     energy in kcal/mol and a boolean value indicating success.
     '''
-
+    reagents, products = align_structures(np.array([reagents, products]))
     first = Atoms(atomnos, positions=reagents)
     last = Atoms(atomnos, positions=products)
 
-    images =  [first]
-    images += [first.copy() for _ in range(n_images)]
-    images += [last]
+    if mep_override is not None:
 
-    neb = DyNEB(images, fmax=0.05, climb=False,  method='eb', scale_fmax=1, allow_shared_calculator=True)
-    neb.interpolate()
+        images = [Atoms(atomnos, positions=coords) for coords in mep_override]
+        neb = DyNEB(images, fmax=0.05, climb=False, method='eb', scale_fmax=1, allow_shared_calculator=True)
 
-    ase_dump(f'{title}_MEP_guess.xyz', images, atomnos)
+    elif ts_guess is None:
+        images =  [first]
+        images += [first.copy() for _ in range(n_images)]
+        images += [last]
 
-    if verbose_print and logfunction is not None:
+        neb = DyNEB(images, fmax=0.05, climb=False, method='eb', scale_fmax=1, allow_shared_calculator=True)
+        neb.interpolate(method='idpp')
+
+    else:
+        ts_guess = Atoms(atomnos, positions=ts_guess)
+
+        images_1 = [first] + [first.copy() for _ in range(round((n_images-3)/2))] + [ts_guess]
+        interp_1 = DyNEB(images_1)
+        interp_1.interpolate(method='idpp')
+
+        images_2 = [ts_guess] + [last.copy() for _ in range(n_images-len(interp_1.images)-1)] + [last]
+        interp_2 = DyNEB(images_2)
+        interp_2.interpolate(method='idpp')
+
+        images = interp_1.images + interp_2.images[1:]
+
+        neb = DyNEB(images, fmax=0.05, climb=False, method='eb', scale_fmax=1, allow_shared_calculator=True)
+
+    if mep_override is None:
+        ase_dump(f'{title}_MEP_guess.xyz', images, atomnos)
+
+    if verbose_print and logfunction is not None and mep_override is None:
         logfunction(f'\n\n--> Saved interpolated MEP guess to {title}_MEP_guess.xyz\n')
     
     # Set calculators for all images
@@ -412,32 +445,56 @@ def ase_neb(embedder, reagents, products, atomnos, n_images=6, title='temp', opt
             # ignore runtime warnings from the NEB module:
             # if something went wrong, we will deal with it later
 
-            with optimizer(neb, maxstep=0.1, logfile=None if not verbose_print else '-') as opt:
+            with optimizer(neb, maxstep=0.1, logfile=None if not verbose_print else 'neb_opt.log') as opt:
+
+                if verbose_print and logfunction is not None:
+                    logfunction(f'\n--> Running NEB-CI through ASE ({embedder.options.theory_level} via {embedder.options.calculator})')
 
                 opt.run(fmax=0.05, steps=20)
-                # some free relaxation before starting to climb
+                while neb.get_residual() > 0.1:
+                    opt.run(fmax=0.05, steps=10+opt.nsteps)
+                    # some free relaxation before starting to climb
+                    if opt.nsteps > 500 or opt.converged:
+                        break
 
+                if verbose_print and logfunction is not None:
+                    logfunction(f'--> fmax below 0.1: Activated Climbing Image and smaller maxstep')
+
+                ase_dump(f'{title}_MEP_start_of_CI.xyz', neb.images, atomnos)
+
+                optimizer.maxstep = 0.01
                 neb.climb = True
-                opt.run(fmax=0.05, steps=500)
+
+                opt.run(fmax=0.05, steps=250+opt.nsteps)
+
                 iterations = opt.nsteps
+                exit_status = 'CONVERGED' if iterations < 279 else 'MAX ITER'
+
+        success = True if exit_status == 'CONVERGED' else False
 
     except (PropertyNotImplementedError, CalculationFailed):
         if logfunction is not None:
             logfunction(f'    - NEB for {title} CRASHED ({time_to_string(time.perf_counter()-t_start)})\n')
-        return None, None, False
+            try:
+                ase_dump(f'{title}_MEP_crashed.xyz', neb.images, atomnos)
+            except Exception():
+                pass
+        return None, None, None, False
 
-    exit_status = 'CONVERGED' if iterations < 499 else 'MAX ITER'
+    except KeyboardInterrupt:
+        exit_status = 'ABORTED BY USER'
 
     if logfunction is not None:
         logfunction(f'    - NEB for {title} {exit_status} ({time_to_string(time.perf_counter()-t_start)})\n')
 
-    energies = [image.get_total_energy() * 23.06054194532933 for image in images]
-    # eV to kcal/mol
+    energies = [image.get_total_energy() * 23.06054194532933 for image in images] # eV to kcal/mol
+    
     ts_id = energies.index(max(energies))
     # print(f'TS structure is number {ts_id}, energy is {max(energies)}')
 
-    os.remove(f'{title}_MEP_guess.xyz')
-    ase_dump(f'{title}_MEP.xyz', images, atomnos)
+    if mep_override is None:
+        os.remove(f'{title}_MEP_guess.xyz')
+    ase_dump(f'{title}_MEP.xyz', images, atomnos, energies)
     # Save the converged MEP (minimum energy path) to an .xyz file
 
     if write_plot:
@@ -466,7 +523,7 @@ def ase_neb(embedder, reagents, products, atomnos, n_images=6, title='temp', opt
         plt.ylabel('Rel. E. (kcal/mol)')
         plt.savefig(f'{title.replace(" ", "_")}_plt.svg')
 
-    return images[ts_id].get_positions(), energies[ts_id], True
+    return images[ts_id].get_positions(), energies[ts_id], energies, exit_status
 
 class OrbitalSpring:
     '''
@@ -474,9 +531,9 @@ class OrbitalSpring:
     Adds a series of forces based on a pair of orbitals, that is
     virtual points "bonded" to a given atom.
 
-    :params i1, i2: indexes of reactive atoms
+    :params i1, i2: indices of reactive atoms
     :params orb1, orb2: 3D coordinates of orbitals
-    :params neighbors_of_1, neighbors_of_2: lists of indexes for atoms bonded to i1/i2
+    :params neighbors_of_1, neighbors_of_2: lists of indices for atoms bonded to i1/i2
     :params d_eq: equilibrium target distance between orbital centers
     '''
     def __init__(self, i1, i2, orb1, orb2, neighbors_of_1, neighbors_of_2, d_eq, k=1000):
@@ -561,7 +618,7 @@ def PreventScramblingConstraint(graph, atoms, double_bond_protection=False, fix_
 
     dihedrals_deg = None
     if double_bond_protection:
-        double_bonds = get_double_bonds_indexes(atoms.positions, atoms.get_atomic_numbers())
+        double_bonds = get_double_bonds_indices(atoms.positions, atoms.get_atomic_numbers())
         if double_bonds != []:
             dihedrals_deg = []
             for a, b in double_bonds:
@@ -576,25 +633,25 @@ def PreventScramblingConstraint(graph, atoms, double_bond_protection=False, fix_
 
     return FixInternals(dihedrals_deg=dihedrals_deg, angles_deg=angles_deg, bonds=bonds, epsilon=1)
 
-def ase_popt(embedder, coords, atomnos, constrained_indexes=None,
+def ase_popt(embedder, coords, atomnos, constrained_indices=None,
              steps=500, targets=None, safe=False, safe_mask=None,
              traj=None, logfunction=None, title='temp'):
     '''
     embedder: TSCoDe embedder object
     coords: 
     atomnos: 
-    constrained_indexes:
+    constrained_indices:
     safe: if True, adds a potential that prevents atoms from scrambling
     safe_mask: bool array, with False for atoms to be excluded when calculating bonds to preserve
     traj: if set to a string, traj is used as a filename for the bending trajectory.
-          not only the atoms will be printed, but also all the orbitals and the active pivot.
+    not only the atoms will be printed, but also all the orbitals and the active pivot.
     '''
     atoms = Atoms(atomnos, positions=coords)
     atoms.calc = get_ase_calc(embedder)
     constraints = []
 
-    if constrained_indexes is not None:
-        for i, c in enumerate(constrained_indexes):
+    if constrained_indices is not None:
+        for i, c in enumerate(constrained_indices):
             i1, i2 = c
             tgt_dist = norm_of(coords[i1]-coords[i2]) if targets is None else targets[i]
             constraints.append(Spring(i1, i2, tgt_dist))
@@ -631,10 +688,10 @@ def ase_bend(embedder, original_mol, conf, pivot, threshold, title='temp', traj=
     pivot: pivot connecting two Hypermolecule orbitals to be approached/distanced
     threshold: target distance for the specified pivot, in Angstroms
     title: name to be used for referring to this structure in the embedder log
-    traj: if set to a string, traj+'.traj' is used as a filename for the bending trajectory.
-          not only the atoms will be printed, but also all the orbitals and the active pivot.
+    traj: if set to a string, traj+\'.traj\' is used as a filename for the bending trajectory.
+    not only the atoms will be printed, but also all the orbitals and the active pivot.
     check: if True, after bending checks that the bent structure did not scramble.
-           If it did, returns the initial molecule.
+    If it did, returns the initial molecule.
     '''
 
     identifier = np.sum(original_mol.atomcoords[conf])
@@ -668,7 +725,7 @@ def ase_bend(embedder, original_mol, conf, pivot, threshold, title='temp', traj=
         except FileNotFoundError:
             pass
 
-    i1, i2 = original_mol.reactive_indexes
+    i1, i2 = original_mol.reactive_indices
 
     neighbors_of_1 = neighbors(original_mol.graph, i1)
     neighbors_of_2 = neighbors(original_mol.graph, i2)
@@ -782,8 +839,8 @@ def ase_bend(embedder, original_mol, conf, pivot, threshold, title='temp', traj=
     reference = np.array(reference)
     targets = np.array(targets)
 
-    r = reference - np.mean(reference[final_mol.reactive_indexes], axis=0)
-    ts = np.array([t - np.mean(t[final_mol.reactive_indexes], axis=0) for t in targets])
+    r = reference - np.mean(reference[final_mol.reactive_indices], axis=0)
+    ts = np.array([t - np.mean(t[final_mol.reactive_indices], axis=0) for t in targets])
 
     output = []
     output.append(r)
@@ -808,8 +865,16 @@ def ase_bend(embedder, original_mol, conf, pivot, threshold, title='temp', traj=
 
     return final_mol
 
-def ase_dump(filename, images, atomnos):
+def ase_dump(filename, images, atomnos, energies=None):
+
+    if energies is None:
+        energies = ["" for _ in images]
+    else:
+        energies = np.array(energies)
+        energies -= np.min(energies)
+
     with open(filename, 'w') as f:
-                for i, image in enumerate(images):
-                    coords = image.get_positions()
-                    write_xyz(coords, atomnos, f, title=f'{filename[:-4]}_image_{i}')
+        for i, (image, energy) in enumerate(zip(images, energies)):
+            e = f" Rel.E = {round(energy, 3)} kcal/mol" if energy != "" else ""
+            coords = image.get_positions()
+            write_xyz(coords, atomnos, f, title=f'STEP {i+1} - {filename[:-4]}_image_{i+1}{e}')
\ No newline at end of file
