@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 '''
-import getpass
 import logging
 import os
 import random
 import re
-import socket
+import sys
 import time
+from getpass import getuser
 from itertools import groupby
 
 import numpy as np
@@ -29,13 +29,15 @@ import numpy as np
 from tscode.__main__ import __version__
 from tscode.algebra import norm_of
 from tscode.embedder_options import Options, OptionSetter, keywords_list
-from tscode.errors import InputError
+from tscode.embeds import _get_monomolecular_reactive_indices
+from tscode.errors import InputError, NoOrbitalError
+from tscode.graph_manipulations import get_sum_graph
 from tscode.hypermolecule_class import Hypermolecule, Pivot
 from tscode.operators import operate
 from tscode.run import RunEmbedding
-from tscode.settings import CALCULATOR, DEFAULT_LEVELS, PROCS
-from tscode.utils import (ase_view, auto_newline, cartesian_product,
-                          clean_directory, time_to_string)
+from tscode.settings import CALCULATOR, DEFAULT_LEVELS, PROCS, THREADS
+from tscode.utils import (ase_view, auto_newline, cartesian_product, graphize,
+                          time_to_string)
 
 
 class Embedder:
@@ -44,7 +46,7 @@ class Embedder:
     options and initialize the calculation
     '''
 
-    def __init__(self, filename, stamp=None):
+    def __init__(self, filename, stamp=None, procs=None, threads=None):
         '''
         Initialize the Embedder object by reading the input filename (.txt).
         Sets the Option dataclass properties to default and then updates them
@@ -62,6 +64,10 @@ class Embedder:
         else:
             self.stamp = stamp
 
+        self.avail_cpus = len(os.sched_getaffinity(0))
+        self.threads = int(threads) if threads is not None else THREADS or self.avail_cpus//4 or 1
+        self.procs = int(procs) if procs is not None else PROCS or 4
+
         try:
             os.remove(f'TSCoDe_{self.stamp}.log')
 
@@ -117,7 +123,8 @@ class Embedder:
             if self.options.debug:
                 for mol in self.objects:
                     if hasattr(mol, 'reactive_atoms_classes_dict'):
-                        mol.write_hypermolecule()
+                        if len(mol.reactive_atoms_classes_dict[0]) > 0:
+                            mol.write_hypermolecule()
                 self.log(f'--> DEBUG: written hypermolecule files ({len(self.objects)})\n')
 
             if self.options.check_structures:
@@ -138,39 +145,46 @@ class Embedder:
         Write banner to log file, containing program and run info
         '''
         banner = '''
-             ▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁
-            ╱───────────────────────────────────╲
-           ╱▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒╲
-       ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒
-    ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒ 
-   ▒░████████╗░██████╗░█████╗░░█████╗░██████╗░███████╗░░░▒
-   ▒░╚══██╔══╝██╔════╝██╔══██╗██╔══██╗██╔══██╗██╔════╝░░░▒ 
-   ▒░░░░██║░░░╚█████╗░██║░░╚═╝██║░░██║██║░░██║█████╗░░░░░▒  
-   ▒░░░░██║░░░░╚═══██╗██║░░██╗██║░░██║██║░░██║██╔══╝░░░░░▒   
-    ▒░░░██║░░░██████╔╝╚█████╔╝╚█████╔╝██████╔╝███████╗░░░▒    
-   ╱ ▒░░╚═╝░░░╚═════╝░░╚════╝░░╚════╝░╚═════╝░╚══════╝░░▒ ╲
-  ╱   ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒   ╲
- ╱    ▒░░╔══════════════════════════════════════════╗░░▒    ╲
- ╲╲    ▒░║  Transition State Conformational Docker  ║░▒    ╱╱   
-  ╲╲   ▒░║        nicolo.tampellini@yale.edu        ║░▒   ╱╱    
-   ╲╲  ▒░║                                          ║░▒  ╱╱         
-    ╲╲ ▒░║     Version    >{0:^25}║░▒ ╱╱                                     
-     ╲╲▒░║      User      >{1:^25}║░▒╱╱                                       
-      ╲▒░║      Host      >{2:^25}║░▒╱                                                     
-       ▒░║      Time      >{3:^25}║░▒                 
-       ▒░╚══════════════════════════════════════════╝░▒                     
-        ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒                             
-          ╲╲ ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒ ╱╱
-           ╲╲▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁╱╱                                        
+       +   .     ▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁ .     .    
+    *    .   .. ╱────────────────────────────────────╲   *     .  
+ .     ..   +  ╱▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒ ╲ .   .   +  
+   +       ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒ . ..   .  
+     .  ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒  .   *  
+       ▒░████████╗░██████╗░█████╗░░█████╗░██████╗░███████╗░░░▒  .  .  
+   +   ▒░╚══██╔══╝██╔════╝██╔══██╗██╔══██╗██╔══██╗██╔════╝░░░▒   ..  
+ ..  . ▒░░░░██║░░░╚█████╗░██║░░╚═╝██║░░██║██║░░██║█████╗░░░░░▒ *    +   
+   .   ▒░░░░██║░░░░╚═══██╗██║░░██╗██║░░██║██║░░██║██╔══╝░░░░░▒   .   .
+.       ▒░░░██║░░░██████╔╝╚█████╔╝╚█████╔╝██████╔╝███████╗░░░▒ ..   +   
+ *  .  ╱ ▒░░╚═╝░░░╚═════╝░░╚════╝░░╚════╝░╚═════╝░╚══════╝░░▒ ╲ .  ..  
+  ..  ╱   ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒   ╲   .   
+.    ╱    ▒░░╔══════════════════════════════════════════╗░░▒    ╲ +    
+    ╱      ▒░║  Transition State Conformational Docker  ║░▒      ╲ ..  
+ +  ╲╲     ▒░║        nicolo.tampellini@yale.edu        ║░▒     ╱╱    .  
+     ╲╲    ▒░║                                          ║░▒    ╱╱  .       
+ ..   ╲╲   ▒░║     Version    >{0:^25}║░▒   ╱╱ .  *                                    
+   .   ╲╲  ▒░║      User      >{1:^25}║░▒  ╱╱   .                                     
+        ╲╲ ▒░║      Time      >{2:^25}║░▒ ╱╱ *   .                                                      
+ ..   *  ╲╲▒░║      Procs     >{3:^25}║░▒╱╱   ..            
+    .     ╲▒░║     Threads    >{4:^25}║░▒╱  +              
+      .    ▒░║    Avail CPUs  >{5:^25}║░▒ .   ..                            
+  +  .. .  ▒░╚══════════════════════════════════════════╝░▒  .. .   
+    .       ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒     .     
+ .     *  +   ╲╲ ▒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░▒ ╱╱  .      .
+     .      .  ╲╲▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁╱╱ .   .    
            '''.format(__version__,
-                      getpass.getuser(),
-                      socket.gethostname(),
-                      time.ctime()[0:-8])
+                      getuser(),
+                      time.ctime()[0:-8],
+                      self.procs,
+                      self.threads,
+                      self.avail_cpus)
 
         # ⏣█▓▒░ banner art adapted from https://fsymbols.com/generators/tarty/
 
         self.log(banner)
 
+        if self.procs * self.threads > self.avail_cpus:
+            self.log(f'--> ATTENTION: Excessive hyperthreading - {self.avail_cpus} CPUs (w/Hyperthreading) detected, {self.procs}*{self.threads} will be used')
+
     def _parse_input(self, filename):
         '''
         Reads a textfile and sets the Embedder properties for the run.
@@ -182,12 +196,12 @@ class Embedder:
         with open(filename, 'r') as f:
             lines = f.readlines()
 
-        lines = [line for line in lines if line[0] not in ('#', '\n')]
+        lines = [line.replace(', ',',') for line in lines if line[0] not in ('#', '\n')]
         
         def _remove_internal_constraints(string):
             numbers = [int(re.sub('[^0-9]', '', i)) for i in string]
             letters = [re.sub('[^a-z]', '', i) for i in string]
-            count = [letters.count(l) for l in letters]
+            count = [letters.count(l) if (l != '') else 1 for l in letters]
             return tuple([n for n, c in zip(numbers, count) if c == 1])
 
         try:
@@ -199,9 +213,10 @@ class Embedder:
                 self.mol_lines = lines
 
             inp = []
-            for line in self.mol_lines:
+            for _l, line in enumerate(self.mol_lines):
 
                 if '>' in line:
+                    self.options.operators_dict[_l] = [op.rstrip().lstrip() for op in reversed(line.rstrip('\n').split('>')[:-1])]
                     self.options.operators.append(line.rstrip('\n'))
                     line = line.split('>')[-1].lstrip()
                     # record that we will need to perform these operations before the run
@@ -209,11 +224,11 @@ class Embedder:
                 filename, *reactive_atoms = line.split()
 
                 if reactive_atoms:
-                    reactive_indexes = _remove_internal_constraints(reactive_atoms)
+                    reactive_indices = _remove_internal_constraints(reactive_atoms)
                 else:
-                    reactive_indexes = None
+                    reactive_indices = None
 
-                inp.append((filename, reactive_indexes))
+                inp.append((filename, reactive_indices))
 
             return inp
             
@@ -289,7 +304,7 @@ class Embedder:
             # appending pairing to dict before
             # calculating their cumulative index
             # If a pairing is already present, add the number
-            # (run>/RUN runs)
+            # (refine>/REFINE runs)
             for index, letter in pairings:
 
                 if self.pairings_dict[i].get(letter) is not None:
@@ -317,7 +332,7 @@ class Embedder:
             for cumulative_pair in pairings:
                 parsed.append(cumulative_pair)
         # parsed looks like [[1, 'a'], [9, 'a']] where numbers are
-        # cumulative indexes for TSs
+        # cumulative indices for TSs
 
         links = {j:[] for j in set([i[1] for i in parsed])}
         for index, tag in parsed:
@@ -357,6 +372,10 @@ class Embedder:
         # Now record the internal constraints, that is the intramolecular
         # distances to freeze and later enforce to the imposed spacings
         self.internal_constraints = []
+
+        # making sure we set the kw_line attribute
+        self.kw_line = self.kw_line if hasattr(self, 'kw_line') else ''
+        
         for letter, pair in self.pairings_table.items():
             for mol_id in self.pairings_dict:
                 if isinstance(self.pairings_dict[mol_id].get(letter), tuple):
@@ -378,11 +397,11 @@ class Embedder:
             if not hasattr(mol, 'reactive_atoms_classes_dict'):
                 mol.compute_orbitals()
 
-        self.pairings_dists = {piece.split('=')[0] : float(piece.split('=')[1]) for piece in orb_string.split(',')}
+        self.pairing_dists = {piece.split('=')[0] : float(piece.split('=')[1]) for piece in orb_string.split(',')}
 
         # Set the new orbital center with imposed distance from the reactive atom. The imposed distance is half the 
         # user-specified one, as the final atomic distances will be given by two halves of this length.
-        for letter, dist in self.pairings_dists.items():
+        for letter, dist in self.pairing_dists.items():
 
             if letter not in self.pairings_table:
                 raise SyntaxError(f'Letter \'{letter}\' is specified in DIST but not present in molecules string.')
@@ -419,26 +438,6 @@ class Embedder:
         mol.pivots = self._get_pivots(mol)
 
         for c, _ in enumerate(mol.atomcoords):
-
-            # if len(mol.pivots[c]) == 2:
-            # # reactive atoms have one and two centers,
-            # # respectively. Apply bridging carboxylic acid correction.
-            #     symbols = [atom.symbol for atom in mol.reactive_atoms_classes_dict[c].values()]
-            #     if 'H' in symbols:
-            #         if ('O' in symbols) or ('S' in symbols):
-            #             if max([norm_of(p.pivot)/self.options.shrink_multiplier for p in mol.pivots[c]]) < 4.5:
-            #                 class_types = [str(atom) for atom in mol.reactive_atoms_classes_dict[c].values()]
-            #                 if 'Single Bond' in class_types and 'Ketone' in class_types:
-            #                 # if we have a bridging acid, remove the longest of the two pivots,
-            #                 # as it would lead to weird structures
-            #                     norms = np.linalg.norm([p.pivot for p in mol.pivots[c]], axis=1)
-            #                     for sample in norms:
-            #                         to_keep = [i for i in norms if sample >= i]
-            #                         if len(to_keep) == 1:
-            #                             mask = np.array([i in to_keep for i in norms])
-            #                             mol.pivots[c] = mol.pivots[c][mask]
-            #                             break
-
             if self.options.suprafacial:
                 if len(mol.pivots[c]) == 4:
                 # reactive atoms have two centers each.
@@ -456,7 +455,7 @@ class Embedder:
             # Bond centers) then remove all pivots that are not the shortest. This ensures
             # the "suprafaciality" to the pivots used, preventing the embed of
             # impossible bonding structures
-            if mol.sp3_sigmastar:
+            if hasattr(mol, 'sp3_sigmastar') and mol.sp3_sigmastar:
                 pivots_lengths = [norm_of(pivot.pivot) for pivot in mol.pivots[c]]
                 shortest_length = min(pivots_lengths)
                 mask = np.array([(i - shortest_length) < 1e-5 for i in pivots_lengths])
@@ -468,6 +467,10 @@ class Embedder:
         (Cyclical embed) Function that yields the molecule pivots. Called by _set_pivots
         and in pre-conditioning (deforming, bending) the molecules in ase_bend.
         '''
+
+        if not hasattr(mol, 'reactive_atoms_classes_dict'):
+            return []
+
         pivots_list = [[] for _ in mol.atomcoords]
 
         for c, _ in enumerate(mol.atomcoords):
@@ -475,11 +478,11 @@ class Embedder:
             if len(mol.reactive_atoms_classes_dict[c]) == 2:
             # most molecules: dienes and alkenes for Diels-Alder, conjugated ketones for acid-bridged additions
 
-                indexes = cartesian_product(*[range(len(atom.center)) for atom in mol.reactive_atoms_classes_dict[c].values()])
-                # indexes of vectors in reactive_atom.center. Reactive atoms are 2 and so for one center on atom 0 and 
+                indices = cartesian_product(*[range(len(atom.center)) for atom in mol.reactive_atoms_classes_dict[c].values()])
+                # indices of vectors in reactive_atom.center. Reactive atoms are 2 and so for one center on atom 0 and 
                 # 2 centers on atom 2 we get [[0,0], [0,1], [1,0], [1,1]]
 
-                for i,j in indexes:
+                for i,j in indices:
                     a1, a2 = mol.get_r_atoms(c)
                    
                     c1 = a1.center[i]
@@ -490,12 +493,12 @@ class Embedder:
             elif len(mol.reactive_atoms_classes_dict[c]) == 1:
             # carbenes, oxygen atom in Prilezhaev reaction, SO2 in classic chelotropic reactions
 
-                indexes = cartesian_product(*[range(len(mol.get_r_atoms(c)[0].center)) for _ in range(2)])
-                indexes = [i for i in indexes if i[0] != i[1] and (sorted(i) == i).all()]
-                # indexes of vectors in reactive_atom.center. Reactive atoms is just one, that builds pivots with itself. 
+                indices = cartesian_product(*[range(len(mol.get_r_atoms(c)[0].center)) for _ in range(2)])
+                indices = [i for i in indices if i[0] != i[1] and (sorted(i) == i).all()]
+                # indices of vectors in reactive_atom.center. Reactive atoms is just one, that builds pivots with itself. 
                 # pivots with the same index or inverse order are discarded. 2 centers on one atom 2 yield just [[0,1]]
                 
-                for i,j in indexes:
+                for i,j in indices:
                     a1 = mol.get_r_atoms(c)[0]
                     # chelotropic embeds have pivots that start/end on the same atom
 
@@ -512,17 +515,17 @@ class Embedder:
         '''
 
         if any('pka>'      in op for op in self.options.operators) or (
-           any('approach>' in op for op in self.options.operators)
+           any('scan>' in op for op in self.options.operators)
         ):
             self.embed = 'data'
-            # If a pka or approach operator is requested, the embed is skipped
+            # If a pka or scan operator is requested, the embed is skipped
             # and data is shown instead
             return
 
-        if any('run>' in op for op in self.options.operators) or self.options.noembed:
-            self.embed = 'run'
+        if any('refine>' in op for op in self.options.operators) or self.options.noembed:
+            self.embed = 'refine'
 
-            # If the run is a run>/RUN one, the self.embed
+            # If the run is a refine>/REFINE one, the self.embed
             # attribute is set in advance by the self._set_options
             # function through the OptionSetter class
             return
@@ -533,10 +536,10 @@ class Embedder:
                 mol.atomcoords = mol.atomcoords[0:self.options.max_confs]
         # remove conformers if there are too many
 
-        if all([len(mol.reactive_indexes) == 0 for mol in self.objects]):
+        if all([len(mol.reactive_indices) == 0 for mol in self.objects]):
             self.embed = None
-            # Flag the embed type as None if no reactive indexes are
-            # provided (and the run is not a run> one)
+            # Flag the embed type as None if no reactive indices are
+            # provided (and the run is not a refine> one)
             return
 
         if len(self.objects) == 1:
@@ -544,7 +547,7 @@ class Embedder:
 
             mol = self.objects[0]
 
-            if len(mol.reactive_indexes) == 4:
+            if len(mol.reactive_indices) == 4:
                 self.embed = 'dihedral'
                 if 'kcal' not in self.kw_line.lower():
                 # set to 5 if user did not specify a value
@@ -556,7 +559,7 @@ class Embedder:
 
                 return
 
-            if len(mol.reactive_indexes) == 2:
+            if len(mol.reactive_indices) == 2:
 
                 self.embed = 'monomolecular'
                 mol.compute_orbitals()
@@ -575,14 +578,23 @@ class Embedder:
         elif len(self.objects) in (2,3):
         # Setting embed type and calculating the number of conformation combinations based on embed type
 
-            cyclical = all([len(molecule.reactive_indexes) == 2 for molecule in self.objects])
-            chelotropic = sorted([len(molecule.reactive_indexes) for molecule in self.objects]) == [1,2]
-            string = all([len(molecule.reactive_indexes) == 1 for molecule in self.objects]) and len(self.objects) == 2
+            cyclical = all(len(molecule.reactive_indices) == 2 for molecule in self.objects)
 
-            if cyclical or chelotropic:
+            # chelotropic embed should check that the two atoms on one molecule are bonded
+            chelotropic = sorted(len(molecule.reactive_indices) for molecule in self.objects) == [1,2]
+
+            string = all(len(molecule.reactive_indices) == 1 for molecule in self.objects) and len(self.objects) == 2
+
+            multiembed = (len(self.objects) == 2 and
+                          all(len(molecule.reactive_indices) >= 2 for molecule in self.objects) and 
+                          not cyclical)
+
+            if cyclical or chelotropic or multiembed:
 
                 if cyclical:
                     self.embed = 'cyclical'
+                elif multiembed:
+                    self.embed = 'multiembed'
                 else:
                     self.embed = 'chelotropic'
                     for mol in self.objects:
@@ -622,14 +634,19 @@ class Embedder:
                     self.options.rotation_steps = self.options.custom_rotation_steps
 
                 self.systematic_angles = [n * 360 / self.options.rotation_steps for n in range(self.options.rotation_steps)]
+
+            elif multiembed:
+                # Complex, unspecified embed type - will explore many possibilities concurrently
+                self.embed = 'multiembed' 
    
             else:
                 raise InputError(('Bad input - The only molecular configurations accepted are:\n' 
                                   '1) One molecule with two reactive centers (monomolecular embed)\n'
-                                  '2) One molecule with four indexes (dihedral embed)\n'
+                                  '2) One molecule with four indices(dihedral embed)\n'
                                   '3) Two or three molecules with two reactive centers each (cyclical embed)\n'
                                   '4) Two molecules with one reactive center each (string embed)\n'
-                                  '5) Two molecules, one with a single reactive center and the other with two (chelotropic embed)'))
+                                  '5) Two molecules, one with a single reactive center and the other with two (chelotropic embed)\n'
+                                  '6) Two molecules with at least two reactive centers each'))
             
             if p:
             # avoid calculating this if this is an early call 
@@ -639,7 +656,7 @@ class Embedder:
                 # number indexing in the TS context
 
         else:
-            raise InputError('Bad input - too many/few molecules specified (one to three required).')
+            raise InputError('Bad input - could not set up an appropriate embed type (too many structures specified?)')
 
         if self.options.shrink:
             for molecule in self.objects:
@@ -649,15 +666,12 @@ class Embedder:
         # SHRINK - scale orbitals and rebuild pivots
 
         if self.options.rmsd is None:
-            self.options.rmsd = 0.5
-
-            if sum(self.ids) < 50:
-                self.options.rmsd = 0.3
-            # small molecules need smaller RMSD threshold
+            self.options.rmsd = 0.25
 
         if p:
             self.candidates = self._get_number_of_candidates()
-            self.log(f'--> Setup performed correctly. {self.candidates} candidates will be generated.\n')
+            _s = self.candidates or 'Many'
+            self.log(f'--> Setup performed correctly. {_s} candidates will be generated.\n')
 
     def _get_number_of_candidates(self):
         '''
@@ -674,6 +688,9 @@ class Embedder:
                                      for conf, _ in enumerate(mol.atomcoords)]) 
                                 for mol in self.objects]))
                       )
+        
+        if self.embed == 'multiembed':
+            return 0
 
         candidates = 2*len(self.systematic_angles)*np.prod([len(mol.atomcoords) for mol in self.objects])
         
@@ -703,6 +720,19 @@ class Embedder:
 
         return int(candidates)
 
+    def _set_embedder_structures_from_mol(self):
+        '''
+        Intended for REFINE runs, set the self.structures variable
+        (and related) to the confomers of a specific molecuele.
+        '''
+        self.structures = self.objects[0].atomcoords
+        self.atomnos = self.objects[0].atomnos
+        self.constrained_indices = _get_monomolecular_reactive_indices(self)
+        self.ids = None
+        self.energies = np.array([0 for _ in self.structures])
+        self.exit_status = np.ones(self.structures.shape[0], dtype=bool)
+        self.embed_graph = get_sum_graph([graphize(self.structures[0], self.atomnos)], self.constrained_indices[0])
+
     def _calculator_setup(self):
         '''
         Set up the calculator to be used with default theory levels.
@@ -724,28 +754,45 @@ class Embedder:
         # early call to get the self.embed attribute
         self._setup(p=False)
 
-        for input_string in self.options.operators:
+        # for input_string in self.options.operators:
+        for index, operators in self.options.operators_dict.items():
 
-            outname = operate(input_string, self)
-            operator = input_string.split('>')[0]
+            for operator in operators: 
 
-            # these operators do not need molecule substitution
-            if operator not in ('run', 'pka', 'approach'):
+                input_string = f'{operator}> {self.objects[index].name}'
+                outname = operate(input_string, self)
+                # operator = input_string.split('>')[0]
 
-                names = [mol.name for mol in self.objects]
-                filename = self._extract_filename(input_string)
-                index = names.index(filename)
-                reactive_indexes = self.objects[index].reactive_indexes
+                if operator == 'refine':
+                    self._set_embedder_structures_from_mol()
 
-                # replacing the old molecule with the one post-operators
-                self.objects[index] = Hypermolecule(outname, reactive_indexes)
+                # these operators do not need molecule substitution
+                elif operator not in ('pka', 'scan'):
 
-                # calculating where the new orbitals are
-                self.objects[index].compute_orbitals()
+                    # names = [mol.name for mol in self.objects]
+                    # filename = self._extract_filename(input_string)
+                    # index = names.index(filename)
+                    reactive_indices = self.objects[index].reactive_indices
 
-                # updating orbital size if not default
-                if hasattr(self, 'orb_string'):
-                    self._set_custom_orbs(self.orb_string)
+                    # replacing the old molecule with the one post-operators
+                    self.objects[index] = Hypermolecule(outname, reactive_indices)
+
+                    # calculating where the new orbitals are
+                    self.objects[index].compute_orbitals()
+
+                    # updating orbital size if not default
+                    if hasattr(self, 'orb_string'):
+                        self._set_custom_orbs(self.orb_string)
+
+                    # updating global docker if necessary
+                    if operator in ('rsearch', 'csearch') and self.options.noembed and len(self.objects) == 1:
+                        self.structures = self.objects[0].atomcoords
+                        self.atomnos = self.objects[0].atomnos
+                        self.constrained_indices = _get_monomolecular_reactive_indices(self)
+                        self.ids = None
+                        self.energies = np.array([0 for _ in self.structures])
+                        self.exit_status = np.ones(self.structures.shape[0], dtype=bool)
+                        self.embed_graph = get_sum_graph([graphize(self.structures[0], self.atomnos)], self.constrained_indices[0])
 
         # updating the orbital cumnums for 
         # all the molecules in the run
@@ -756,7 +803,7 @@ class Embedder:
 
     def _extract_filename(self, input_string):
         '''
-        Input: 'run> TSCoDe_unoptimized_comp_check.xyz 5a 36a 0b 43b 33c 60c'
+        Input: 'refine> TSCoDe_unoptimized_comp_check.xyz 5a 36a 0b 43b 33c 60c'
         Output: 'TSCoDe_unoptimized_comp_check.xyz'
         '''
         input_string = input_string.split('>')[-1].lstrip()
@@ -779,17 +826,65 @@ class Embedder:
         self.logfile.close()
         os.remove(f'TSCoDe_{self.stamp}.log')
 
-        quit()
+        sys.exit()
 
     def scramble(self, array, sequence):
         return np.array([array[s] for s in sequence])
 
-    def normal_termination(self):
-        clean_directory()
-        self.log(f'\n--> TSCoDe normal termination: total time {time_to_string(time.perf_counter() - self.t_start_run, verbose=True)}.')
-        self.write_quote()
-        self.logfile.close()
-        quit()
+    def get_pairing_dist_from_letter(self, letter):
+        '''
+        Get constrained distance between paired reactive
+        atoms, accessed via the associated constraint letter
+        '''
+
+        if hasattr(self, 'pairing_dists') and self.pairing_dists.get(letter) is not None:
+            # if self.options.shrink:
+            #     return self.pairing_dists[letter] * self.options.shrink_multiplier
+            return self.pairing_dists[letter]
+
+        d = 0
+        try:
+            for mol_index, mol_pairing_dict in self.pairings_dict.items():
+                if r_atom_index := mol_pairing_dict.get(letter):
+
+                    # for refine embeds, one letter corresponds to two indices
+                    # on the same molecule
+                    if isinstance(r_atom_index, tuple):
+                        i1, i2 = r_atom_index
+                        return (self.objects[mol_index].get_orbital_length(i1) +
+                                self.objects[mol_index].get_orbital_length(i2))
+
+                    # for other runs, it is just one atom per molecule per letter
+                    d += self.objects[mol_index].get_orbital_length(r_atom_index)
+
+            return d
+
+        # If no orbitals were built, return None
+        except NoOrbitalError:
+            return None
+
+    def get_pairing_dists_from_constrained_indices(self, constrained_pair):
+        '''
+        Returns the constrained distance
+        for a specific constrained pair of indices
+        '''
+        try:
+            letter = next(lett for lett, pair in self.pairings_table.items() if (pair[0] == constrained_pair[0] and      
+                                                                                 pair[1] == constrained_pair[1]))
+            return self.get_pairing_dist_from_letter(letter)
+
+        except StopIteration:
+            return None
+
+    def get_pairing_dists(self, conf):
+        '''
+        Returns a list with the constrained distances for each embedder constraint
+        '''
+        if self.constrained_indices[conf].size == 0:
+            return None
+
+        constraints = np.concatenate([self.constrained_indices[conf], self.internal_constraints]) if len(self.internal_constraints) > 0 else self.constrained_indices[conf]
+        return [self.get_pairing_dists_from_constrained_indices(pair) for pair in constraints]
 
     def write_quote(self):
         '''
@@ -810,6 +905,17 @@ class Embedder:
         try:
             RunEmbedding(self).run()
 
-        except Exception as e:
-            logging.exception(e)
-            raise e
+        except Exception as _e:
+            logging.exception(_e)
+            raise _e
+        
+    def normal_termination(self):
+        '''
+        Terminate the run, printing the total time and a quote.
+
+        '''
+        self.log(f'\n--> TSCoDe normal termination: total time {time_to_string(time.perf_counter() - self.t_start_run, verbose=True)}.')
+        
+        self.write_quote()
+        self.logfile.close()
+        sys.exit()
\ No newline at end of file
