@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ import numpy as np
 from tscode.algebra import (align_vec_pair, norm, rot_mat_from_pointer,
                             vec_angle)
 from tscode.ase_manipulations import ase_bend
-from tscode.clustered_csearch import _get_quadruplets
+from tscode.torsion_module import _get_quadruplets
 from tscode.errors import TriangleError, ZeroCandidatesError
 from tscode.graph_manipulations import get_sum_graph
 from tscode.optimization_methods import optimize
@@ -39,7 +39,7 @@ def string_embed(embedder):
     '''
     assert len(embedder.objects) == 2
 
-    # if embedder.options.tscode_procs > 1:
+    # if embedder.options.threads > 1:
     #     return string_embed_parallel(embedder)
     # Not implemented for now - significatively slower
 
@@ -70,30 +70,30 @@ def string_embed(embedder):
     embedder.log(f'\n--> Performing string embed ({pretty_num(embedder.candidates)} candidates)')
 
     conf_number = [len(mol.atomcoords) for mol in embedder.objects]
-    conf_indexes = cartesian_product(*[np.array(range(i)) for i in conf_number])
+    conf_indices = cartesian_product(*[np.array(range(i)) for i in conf_number])
     # (n,2) vectors where the every element is the conformer index for that molecule
 
-    r_atoms_centers_indexes = cartesian_product(*[np.array(range(len(mol.get_centers(0)[0]))) for mol in embedder.objects])
+    r_atoms_centers_indices = cartesian_product(*[np.array(range(len(mol.get_centers(0)[0]))) for mol in embedder.objects])
     # for two mols with 3 and 2 centers: [[0 0][0 1][1 0][1 1][2 0][2 1]]
 
     # explicit individual molecules
     mol1, mol2 = embedder.objects
 
     # get quadruplets needed for the tfd similarity check
-    constrained_indexes = [[int(embedder.objects[0].reactive_indexes[0]),
-                            int(embedder.objects[1].reactive_indexes[0] + embedder.ids[0])]]
+    constrained_indices = [[int(embedder.objects[0].reactive_indices[0]),
+                            int(embedder.objects[1].reactive_indices[0] + embedder.ids[0])]]
 
-    quadruplets = _get_quadruplets(get_sum_graph((mol1.graph, mol2.graph), constrained_indexes))
+    quadruplets = _get_quadruplets(get_sum_graph((mol1.graph, mol2.graph), constrained_indices))
 
     lru_cache = []
     poses = []
-    for i, (c1, c2) in enumerate(conf_indexes):
+    for i, (c1, c2) in enumerate(conf_indices):
 
-        loadbar(i, len(conf_indexes),
+        loadbar(i, len(conf_indices),
                 prefix=f'Embedding structures ',
                 suffix=f'({len(poses)} found)')
 
-        for ai1, ai2 in r_atoms_centers_indexes:
+        for ai1, ai2 in r_atoms_centers_indices:
             for angle in embedder.systematic_angles:
 
                 ra1 = mol1.get_r_atoms(c1)[0]
@@ -127,7 +127,7 @@ def string_embed(embedder):
         embedder.log(s, p=False)
         raise ZeroCandidatesError(s)
 
-    embedder.constrained_indexes = _get_string_constrained_indexes(embedder, len(poses))
+    embedder.constrained_indices = _get_string_constrained_indices(embedder, len(poses))
 
     return np.array(poses)
 
@@ -137,10 +137,10 @@ def string_embed_parallel(embedder):
     Algorithm used is the "string" algorithm (see docs).
     '''
 
-    def string_embed_thread(mol1, mol2, c1, c2, r_atoms_centers_indexes, angles, i):
+    def string_embed_thread(mol1, mol2, c1, c2, r_atoms_centers_indices, angles, i):
 
         poses = []
-        for ai1, ai2 in r_atoms_centers_indexes:
+        for ai1, ai2 in r_atoms_centers_indices:
             for angle in angles:
 
                 ra1 = mol1.get_r_atoms(c1)[0]
@@ -164,7 +164,7 @@ def string_embed_parallel(embedder):
                 if compenetration_check(embedded_structure, ids=embedder.ids, thresh=embedder.options.clash_thresh):
                     poses.append(embedded_structure)
 
-        print(f'Completed string embedding of conf_indexes {i}')
+        print(f'Completed string embedding of conf_indices {i}')
 
         return poses
 
@@ -172,23 +172,23 @@ def string_embed_parallel(embedder):
 
 
 
-    embedder.log(f'\n--> Performing string embed ({pretty_num(embedder.candidates)} candidates, parallel on up to {embedder.options.procs} cores)')
+    embedder.log(f'\n--> Performing string embed ({pretty_num(embedder.candidates)} candidates, parallel on up to {embedder.procs} cores)')
 
     conf_number = [len(mol.atomcoords) for mol in embedder.objects]
-    conf_indexes = cartesian_product(*[np.array(range(i)) for i in conf_number])
+    conf_indices = cartesian_product(*[np.array(range(i)) for i in conf_number])
     # (n,2) vectors where the every element is the conformer index for that molecule
 
-    r_atoms_centers_indexes = cartesian_product(*[np.array(range(len(mol.get_centers(0)[0]))) for mol in embedder.objects])
+    r_atoms_centers_indices = cartesian_product(*[np.array(range(len(mol.get_centers(0)[0]))) for mol in embedder.objects])
     # for two mols with 3 and 2 centers: [[0 0][0 1][1 0][1 1][2 0][2 1]]
 
     mol1, mol2 = embedder.objects
 
     poses, processes = [], []
-    with ProcessPoolExecutor(max_workers=embedder.options.tscode_procs) as executor:
+    with ProcessPoolExecutor(max_workers=embedder.options.threads) as executor:
         
-        for i, (c1, c2) in enumerate(conf_indexes):
+        for i, (c1, c2) in enumerate(conf_indices):
 
-            # loadbar(i, len(conf_indexes),
+            # loadbar(i, len(conf_indices),
             #         prefix=f'Embedding structures ',
             #         suffix=f'({len(poses)} found)')
 
@@ -196,7 +196,7 @@ def string_embed_parallel(embedder):
                     string_embed_thread,
                     mol1, mol2,
                     c1, c2,
-                    r_atoms_centers_indexes,
+                    r_atoms_centers_indices,
                     embedder.systematic_angles,
                     i
                 )
@@ -205,7 +205,7 @@ def string_embed_parallel(embedder):
         for p in processes:
             poses.extend(p.result())
 
-        # loadbar(len(conf_indexes), len(conf_indexes),
+        # loadbar(len(conf_indices), len(conf_indices),
         #         prefix=f'Embedding structures ',
         #         suffix=f'({len(poses)} found)')
 
@@ -216,19 +216,19 @@ def string_embed_parallel(embedder):
         embedder.log(s, p=False)
         raise ZeroCandidatesError(s)
 
-    embedder.constrained_indexes = _get_string_constrained_indexes(embedder, len(poses))
+    embedder.constrained_indices = _get_string_constrained_indices(embedder, len(poses))
 
     return np.array(poses)
 
-def _get_string_constrained_indexes(embedder, n):
+def _get_string_constrained_indices(embedder, n):
     '''
-    Get constrained indexes referring to the transition states, repeated n times.
+    Get constrained indices referring to the transition states, repeated n times.
     :params n: int
     :return: list of lists consisting in atomic pairs to be constrained.
     '''
     # Two molecules, string algorithm, one constraint for all, repeated n times
-    return np.array([[[int(embedder.objects[0].reactive_indexes[0]),
-                       int(embedder.objects[1].reactive_indexes[0] + embedder.ids[0])]] for _ in range(n)])
+    return np.array([[[int(embedder.objects[0].reactive_indices[0]),
+                       int(embedder.objects[1].reactive_indices[0] + embedder.ids[0])]] for _ in range(n)])
 
 def cyclical_embed(embedder):
     '''
@@ -251,9 +251,9 @@ def cyclical_embed(embedder):
             return np.array([[0, 1,0],
                              [0,-1,0]])
 
-        vertexes = np.zeros((3,2))
+        vertices = np.zeros((3,2))
 
-        vertexes[1] = np.array([norms[0],0])
+        vertices[1] = np.array([norms[0],0])
 
         a = np.power(norms[0], 2)
         b = np.power(norms[1], 2)
@@ -261,20 +261,20 @@ def cyclical_embed(embedder):
         x = (a-b+c)/(2*a**0.5)
         y = (c-x**2)**0.5
 
-        vertexes[2] = np.array([x,y])
+        vertices[2] = np.array([x,y])
         # similar to the code from polygonize, to get the active triangle
         # but without the orientation specified in the polygonize function
         
-        a = vertexes[1,0] # first point, x
-        b = vertexes[2,0] # second point, x
-        c = vertexes[2,1] # second point, y
+        a = vertices[1,0] # first point, x
+        b = vertices[2,0] # second point, x
+        c = vertices[2,1] # second point, y
 
         x = a/2
         y = (b**2 + c**2 - a*b)/(2*c)
         cc = np.array([x,y])
         # 2D coordinates of the triangle circocenter
 
-        v0, v1, v2 = vertexes
+        v0, v1, v2 = vertices
 
         meanpoint1 = np.mean((v0,v1), axis=0)
         meanpoint2 = np.mean((v1,v2), axis=0)
@@ -310,7 +310,7 @@ def cyclical_embed(embedder):
 
         return np.vstack((dir1, dir2, dir3))
 
-    def _adjust_directions(embedder, directions, constrained_indexes, triangle_vectors, pivots, conf_ids):
+    def _adjust_directions(embedder, directions, constrained_indices, triangle_vectors, pivots, conf_ids):
         '''
         For trimolecular TSs, correct molecules pre-alignment. That is, after the initial estimate
         based on pivot triangle circocentrum, systematically rotate each molecule around its pivot
@@ -325,10 +325,10 @@ def cyclical_embed(embedder):
         p0, p1, p2 = [end - start for start, end in triangle_vectors]
         p0_mean, p1_mean, p2_mean = [np.mean((end, start), axis=0) for start, end in triangle_vectors]
 
-        ############### get triangle vertexes
+        ############### get triangle vertices
 
-        vertexes = np.zeros((3,2))
-        vertexes[1] = np.array([norms[0],0])
+        vertices = np.zeros((3,2))
+        vertices[1] = np.array([norms[0],0])
 
         a = np.power(norms[0], 2)
         b = np.power(norms[1], 2)
@@ -336,20 +336,20 @@ def cyclical_embed(embedder):
         x = (a-b+c)/(2*a**0.5)
         y = (c-x**2)**0.5
 
-        vertexes[2] = np.array([x,y])
+        vertices[2] = np.array([x,y])
         # similar to the code from polygonize, to get the active triangle
         # but without the orientation specified in the polygonize function
         
-        a = vertexes[1,0] # first point, x
-        b = vertexes[2,0] # second point, x
-        c = vertexes[2,1] # second point, y
+        a = vertices[1,0] # first point, x
+        b = vertices[2,0] # second point, x
+        c = vertices[2,1] # second point, y
 
         x = a/2
         y = (b**2 + c**2 - a*b)/(2*c)
-        cc = np.array([x,y])
+        # cc = np.array([x,y])
         # 2D coordinates of the triangle circocenter
 
-        v0, v1, v2 = vertexes
+        v0, v1, v2 = vertices
 
         v0 = np.concatenate((v0, [0]))
         v1 = np.concatenate((v1, [0]))
@@ -361,7 +361,7 @@ def cyclical_embed(embedder):
 
             start, end = triangle_vectors[i]
 
-            mol_direction = pivots[i].meanpoint - np.mean(embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indexes], axis=0)
+            mol_direction = pivots[i].meanpoint - np.mean(embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indices], axis=0)
             if np.all(mol_direction == 0.):
                 mol_direction = pivots[i].meanpoint
 
@@ -371,8 +371,8 @@ def cyclical_embed(embedder):
 
         ############### set up pairings between reactive atoms
 
-        pairings = [[None, None] for _ in constrained_indexes]
-        for i, c in enumerate(constrained_indexes):
+        pairings = [[None, None] for _ in constrained_indices]
+        for i, c in enumerate(constrained_indices):
             for m, mol in enumerate(embedder.objects):
                 for index, r_atom in mol.reactive_atoms_classes_dict[0].items():
                     if r_atom.cumnum == c[0]:
@@ -467,18 +467,18 @@ def cyclical_embed(embedder):
         # Do not keep more than 10 conformations, unless LET keyword is provided
 
     conf_number = [len(mol.atomcoords) for mol in embedder.objects]
-    conf_indexes = cartesian_product(*[np.array(range(i)) for i in conf_number])
+    conf_indices = cartesian_product(*[np.array(range(i)) for i in conf_number])
 
     poses = []
-    constrained_indexes = []
-    for ci, conf_ids in enumerate(conf_indexes):
+    constrained_indices = []
+    for ci, conf_ids in enumerate(conf_indices):
 
-        pivots_indexes = cartesian_product(*[range(len(mol.pivots[conf_ids[i]])) for i, mol in enumerate(embedder.objects)])
-        # indexes of pivots in each molecule self.pivots[conf] list. For three mols with 2 pivots each: [[0,0,0], [0,0,1], [0,1,0], ...]
+        pivots_indices = cartesian_product(*[range(len(mol.pivots[conf_ids[i]])) for i, mol in enumerate(embedder.objects)])
+        # indices of pivots in each molecule self.pivots[conf] list. For three mols with 2 pivots each: [[0,0,0], [0,0,1], [0,1,0], ...]
         
-        for p, pi in enumerate(pivots_indexes):
+        for p, pi in enumerate(pivots_indices):
 
-            loadbar(p+ci*(len(pivots_indexes)), len(pivots_indexes)*len(conf_indexes), prefix=f'Embedding structures ')
+            loadbar(p+ci*(len(pivots_indices)), len(pivots_indices)*len(conf_indices), prefix=f'Embedding structures ')
             
             pivots = [embedder.objects[m].pivots[conf_ids[m]][pi[m]] for m, _ in enumerate(embedder.objects)]
             # getting the active pivot for each molecule for this run
@@ -522,7 +522,7 @@ def cyclical_embed(embedder):
                     index = deltas.index(max(deltas))
                     mol = embedder.objects[index]
 
-                    if not tuple(sorted(mol.reactive_indexes)) in list(mol.graph.edges):
+                    if tuple(sorted(mol.reactive_indices)) not in list(mol.graph.edges):
                         # do not try to bend molecules where the two reactive indices are bonded
 
                         pivot = pivots[index]
@@ -595,10 +595,10 @@ def cyclical_embed(embedder):
                 
                     for i, mol in enumerate(deepcopy(embedder.objects)):
 
-                        if len(mol.reactive_indexes) > 1:
+                        if len(mol.reactive_indices) > 1:
                         # do not try to bend molecules that react with a single atom
 
-                            if tuple(sorted(mol.reactive_indexes)) not in list(mol.graph.edges):
+                            if tuple(sorted(mol.reactive_indices)) not in list(mol.graph.edges):
                             # do not try to bend molecules where the two reactive indices are bonded
 
                                 traj = f'bend_{mol.name}_p{p}_tgt_{round(target_length, 3)}' if embedder.options.debug else None
@@ -631,12 +631,12 @@ def cyclical_embed(embedder):
             # directions to orient the molecules toward, orthogonal to each vec_pair
 
             for v, vecs in enumerate(polygon_vectors):
-            # getting vertexes to embed molecules with and iterating over start/end points
+            # getting vertices to embed molecules with and iterating over start/end points
 
-                ids = _get_cyclical_reactive_indexes(embedder, pivots, v)
-                # get indexes of atoms that face each other
+                ids = _get_cyclical_reactive_indices(embedder, pivots, v)
+                # get indices of atoms that face each other
 
-                if not embedder.pairings_table or all([pair in ids for pair in embedder.pairings_table.values()]):
+                if not embedder.pairings_table or all((pair in ids) or (pair in embedder.internal_constraints) for pair in embedder.pairings_table.values()):
                 # ensure that the active arrangement has all the pairings that the user specified
 
                     if len(embedder.objects) == 3:
@@ -656,7 +656,7 @@ def cyclical_embed(embedder):
                             start, end = vec_pair
                             angle = angles[i]
 
-                            reactive_coords = embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indexes]
+                            reactive_coords = embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indices]
                             # coordinates for the reactive atoms in this run
 
                             atomic_pivot_mean = np.mean(reactive_coords, axis=0)
@@ -706,12 +706,12 @@ def cyclical_embed(embedder):
                         if compenetration_check(embedded_structure, ids=embedder.ids, thresh=embedder.options.clash_thresh):
 
                             poses.append(embedded_structure)
-                            constrained_indexes.append(ids)
-                            # Save indexes to be constrained later in the optimization step
+                            constrained_indices.append(ids)
+                            # Save indices to be constrained later in the optimization step
 
     loadbar(1, 1, prefix=f'Embedding structures ')
 
-    embedder.constrained_indexes = np.array(constrained_indexes)
+    embedder.constrained_indices = np.array(constrained_indices)
 
     if not poses:
         s = ('\n--> Cyclical embed did not find any suitable disposition of molecules.\n' +
@@ -731,18 +731,18 @@ def _fast_bimol_rigid_cyclical_embed(embedder):
     embedder.log(f'\n--> Performing {embedder.embed} embed ({embedder.candidates} candidates)')
 
     conf_number = [len(mol.atomcoords) for mol in embedder.objects]
-    conf_indexes = cartesian_product(*[np.array(range(i)) for i in conf_number])
+    conf_indices = cartesian_product(*[np.array(range(i)) for i in conf_number])
 
     poses = []
-    constrained_indexes = []
-    for ci, conf_ids in enumerate(conf_indexes):
+    constrained_indices = []
+    for ci, conf_ids in enumerate(conf_indices):
 
-        pivots_indexes = cartesian_product(*[range(len(mol.pivots[conf_ids[i]])) for i, mol in enumerate(embedder.objects)])
-        # indexes of pivots in each molecule self.pivots[conf] list. For three mols with 2 pivots each: [[0,0,0], [0,0,1], [0,1,0], ...]
-        
-        for p, pi in enumerate(pivots_indexes):
+        pivots_indices = cartesian_product(*[range(len(mol.pivots[conf_ids[i]])) for i, mol in enumerate(embedder.objects)])
+        # indices of pivots in each molecule self.pivots[conf] list. For three mols with 2 pivots each: [[0,0,0], [0,0,1], [0,1,0], ...]
 
-            loadbar(p+ci*(len(pivots_indexes)), len(pivots_indexes)*len(conf_indexes), prefix=f'Embedding structures ')
+        for p, pi in enumerate(pivots_indices):
+
+            loadbar(p+ci*(len(pivots_indices)), len(pivots_indices)*len(conf_indices), prefix=f'Embedding structures ')
             
             pivots = [embedder.objects[m].pivots[conf_ids[m]][pi[m]] for m, _ in enumerate(embedder.objects)]
             # getting the active pivot for each molecule for this run
@@ -757,15 +757,15 @@ def _fast_bimol_rigid_cyclical_embed(embedder):
             polygon_vectors = polygonize(norms)
 
             directions = np.array([[0, 1,0], [0,-1,0]])
-            # directions to orient the molecules toward, orthogonal to each vec_pair
+            # directions to orient the molecules toward, orthogonal to each vec_pair          
 
             for v, vecs in enumerate(polygon_vectors):
-            # getting vertexes to embed molecules with and iterating over start/end points
+            # getting vertices to embed molecules with and iterating over start/end points
 
-                ids = _get_cyclical_reactive_indexes(embedder, pivots, v)
-                # get indexes of atoms that face each other
+                ids = _get_cyclical_reactive_indices(embedder, pivots, v)
+                # get indices of atoms that face each other
 
-                if not embedder.pairings_table or all([pair in ids for pair in embedder.pairings_table.values()]):
+                if not embedder.pairings_table or all((pair in ids) or (pair in embedder.internal_constraints) for pair in embedder.pairings_table.values()):
                 # ensure that the active arrangement has all the pairings that the user specified
                         
                     for angles in embedder.systematic_angles:
@@ -778,7 +778,7 @@ def _fast_bimol_rigid_cyclical_embed(embedder):
                             start, end = vec_pair
                             angle = angles[i]
 
-                            reactive_coords = embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indexes]
+                            reactive_coords = embedder.objects[i].atomcoords[conf_ids[i]][embedder.objects[i].reactive_indices]
                             # coordinates for the reactive atoms in this run
 
                             atomic_pivot_mean = np.mean(reactive_coords, axis=0)
@@ -828,12 +828,12 @@ def _fast_bimol_rigid_cyclical_embed(embedder):
                         if compenetration_check(embedded_structure, ids=embedder.ids, thresh=embedder.options.clash_thresh):
 
                             poses.append(embedded_structure)
-                            constrained_indexes.append(ids)
-                            # Save indexes to be constrained later in the optimization step
+                            constrained_indices.append(ids)
+                            # Save indices to be constrained later in the optimization step
 
     loadbar(1, 1, prefix=f'Embedding structures ')
 
-    embedder.constrained_indexes = np.array(constrained_indexes)
+    embedder.constrained_indices = np.array(constrained_indices)
 
     if not poses:
         s = ('\n--> Cyclical embed did not find any suitable disposition of molecules.\n' +
@@ -844,7 +844,7 @@ def _fast_bimol_rigid_cyclical_embed(embedder):
 
     return np.array(poses)
 
-def _get_cyclical_reactive_indexes(embedder, pivots, n):
+def _get_cyclical_reactive_indices(embedder, pivots, n):
     '''
     :params n: index of the n-th disposition of vectors yielded by the polygonize function.
     :return: list of index couples, to be constrained during the partial optimization.
@@ -927,18 +927,18 @@ def monomolecular_embed(embedder):
     embedder.exit_status = np.zeros(len(embedder.structures), dtype=bool)
     embedder.graphs = [mol.graph]
 
-    embedder.constrained_indexes = _get_monomolecular_reactive_indexes(embedder)
+    embedder.constrained_indices = _get_monomolecular_reactive_indices(embedder)
 
     return embedder.structures
 
-def _get_monomolecular_reactive_indexes(embedder):
+def _get_monomolecular_reactive_indices(embedder):
     '''
     '''
     if embedder.pairings_table:
-            return np.array([list(embedder.pairings_table.values())
-                            for _ in embedder.structures])
+        return np.array([list(embedder.pairings_table.values())
+                        for _ in embedder.structures])
     # This option gives the possibility to specify pairings in
-    # run>/RUN runs, so as to make constrained optimizations
+    # refine>/REFINE runs, so as to make constrained optimizations
     # accessible.
 
     return np.array([[] for _ in embedder.structures])
@@ -951,7 +951,7 @@ def dihedral_embed(embedder):
     embedder.structures, embedder.energies = [], []
 
 
-    embedder.log(f'\n--> {mol.name} - performing a scan of dihedral angle with indices {mol.reactive_indexes}\n')
+    embedder.log(f'\n--> {mol.name} - performing a scan of dihedral angle with indices {mol.reactive_indices}\n')
 
     for c, coords in enumerate(mol.atomcoords):
 
@@ -965,18 +965,20 @@ def dihedral_embed(embedder):
                                                     mol.atomnos,
                                                     embedder.options.calculator,
                                                     method=embedder.options.theory_level,
-                                                    procs=embedder.options.procs,
+                                                    procs=embedder.procs,
                                                     solvent=embedder.options.solvent
                                                 )
 
         if not success:
             embedder.log(f'Pre-optimization failed - Skipped conformer {c+1}', p=False)
+            # with open(f'conf_{c+1}_opt_failed.xyz', 'w') as f:
+            #     write_xyz(new_coords, mol.atomnos, f)
             continue
 
         structures, energies = ase_torsion_TSs(embedder,
                                                 new_coords,
                                                 mol.atomnos,
-                                                mol.reactive_indexes,
+                                                mol.reactive_indices,
                                                 threshold_kcal=embedder.options.kcal_thresh,
                                                 title=mol.rootname+f'_conf_{c+1}',
                                                 optimization=embedder.options.optimization,
@@ -1000,16 +1002,16 @@ def dihedral_embed(embedder):
 
     embedder.atomnos = mol.atomnos
     embedder.similarity_refining()
-    embedder.write_structures('poses', indexes=embedder.objects[0].reactive_indexes, relative=False, extra='(barrier height)')
-    embedder.write_vmd(indexes=embedder.objects[0].reactive_indexes)
+    embedder.write_structures('poses', indices=embedder.objects[0].reactive_indices, relative=False, extra='(barrier height)')
+    # embedder.write_vmd(indices=embedder.objects[0].reactive_indices)
     embedder.normal_termination()
 
 def get_embed(mols, conf_ids):
     '''
     mols: iterable of Hypermolecule objects
-    conf_ids: iterable of conformer indexes for each mol
+    conf_ids: iterable of conformer indices for each mol
 
     Returns an np.array with the coordinates
     of every molecule as a concatenated array.
     '''
-    return np.concatenate([(mol.rotation @ mol.atomcoords[c].T).T + mol.position for mol, c in zip(mols, conf_ids)])
+    return np.concatenate([(mol.rotation @ mol.atomcoords[c].T).T + mol.position for mol, c in zip(mols, conf_ids)])
\ No newline at end of file
