@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,8 +35,9 @@ def graphize(coords, atomnos, mask=None):
     :params coords: atomic coordinates as 3D vectors
     :params atomnos: atomic numbers as a list
     :params mask: bool array, with False for atoms
-                  to be excluded in the bond evaluation
+    to be excluded in the bond evaluation
     :return connectivity graph
+    
     '''
 
     mask = np.array([True for _ in atomnos], dtype=bool) if mask is None else mask
@@ -85,7 +86,7 @@ def get_sp_n(index, graph):
 
     d = {
         6:{2:1, 3:2, 4:3},      # C - 2 neighbors means sp, 3 nb means sp2, 4 nb sp3
-        7:{2:2, 3:3, 4:3},      # N - 2 neighbors means sp2, 3 nb means sp3, 4 nb still sp3
+        7:{2:2, 3:None, 4:3},      # N - 2 neighbors means sp2, 3 nb could mean sp3 or sp2, 4 nb sp3
         8:{1:2, 2:3, 3:3, 4:3}, # O
         15:{2:2, 3:3, 4:3},     # P - like N
         16:{2:2, 3:3, 4:3},     # S
@@ -182,8 +183,8 @@ def get_phenyls(coords, atomnos):
 
     output = []
 
-    c_n_indexes = np.fromiter((i for i, a in enumerate(atomnos) if a in (6,7)), dtype=atomnos.dtype)
-    comb = combinations(c_n_indexes, 6)
+    c_n_indices = np.fromiter((i for i, a in enumerate(atomnos) if a in (6,7)), dtype=atomnos.dtype)
+    comb = combinations(c_n_indices, 6)
 
     for c in comb:
         mask = np.fromiter((i in c for i in range(len(atomnos))), dtype=bool)
@@ -193,6 +194,21 @@ def get_phenyls(coords, atomnos):
 
     return np.array(output)
 
+def _get_phenyl_ids(i, G):
+    '''
+    If i is part of a phenyl group, return the six
+    heavy atom indices associated with the ring
+    '''
+    for n in neighbors(G, i):
+        paths = nx.all_simple_paths(G, source=i, target=n, cutoff=6)
+        for path in paths:
+            if len(path) == 6:
+                if all(G.nodes[n]['atomnos'] != 1 for n in path):
+                    if all(len(neighbors(G, i)) == 3 for i in path):
+                        return path
+    
+    return None
+
 def findPaths(G, u, n, excludeSet = None):
     '''
     Recursively find all paths of a NetworkX
@@ -221,7 +237,7 @@ def is_sigmatropic(mol, conf):
     - has 2 reactive atoms
     - they are of sp2 or analogous types
     - they are connected, or at least one path connecting them
-      is made up of atoms that do not make more than three bonds each
+    is made up of atoms that do not make more than three bonds each
     - they are less than 3 A apart (cisoid propenal makes it, transoid does not)
 
     Used to set the mol.sigmatropic attribute, that affects orbital
@@ -234,9 +250,9 @@ def is_sigmatropic(mol, conf):
                 'sp',
                 'bent carbene'
                 )
-    if len(mol.reactive_indexes) == 2:
+    if len(mol.reactive_indices) == 2:
 
-        i1, i2 = mol.reactive_indexes
+        i1, i2 = mol.reactive_indices
         if norm_of(mol.atomcoords[conf][i1] - mol.atomcoords[conf][i2]) < 3:
 
             if all([str(r_atom) in sp2_types for r_atom in mol.reactive_atoms_classes_dict[conf].values()]):
@@ -271,9 +287,9 @@ def is_vicinal(mol):
                 'Single Bond',
                 )
 
-    if len(mol.reactive_indexes) == 2:
+    if len(mol.reactive_indices) == 2:
 
-        i1, i2 = mol.reactive_indexes
+        i1, i2 = mol.reactive_indices
 
         if all([str(r_atom) in vicinal_types for r_atom in mol.reactive_atoms_classes_dict[0].values()]):
             if i1 in neighbors(mol.graph, i2):
@@ -281,7 +297,7 @@ def is_vicinal(mol):
 
     return False
 
-def get_sum_graph(graphs, extra_edges):
+def get_sum_graph(graphs, extra_edges=None):
     '''
     Creates a graph containing all graphs, added in 
     sequence, and then adds the specified extra edges
@@ -290,13 +306,19 @@ def get_sum_graph(graphs, extra_edges):
 
     graph, *extra = graphs
     out = deepcopy(graph)
+    cum_atomnos = list(nx.get_node_attributes(graphs[0], "atomnos").values())
 
     for g in extra:
         n = len(out.nodes())
         for e1, e2 in g.edges():
             out.add_edge(e1+n, e2+n)
 
-    for e1, e2 in extra_edges:
-        out.add_edge(e1, e2)
+        cum_atomnos += list(nx.get_node_attributes(g, "atomnos").values())
 
-    return graph
+    if extra_edges is not None:
+        for e1, e2 in extra_edges:
+            out.add_edge(e1, e2)
+
+    nx.set_node_attributes(out, dict(enumerate(cum_atomnos)), 'atomnos')
+
+    return out
