@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@ from copy import deepcopy
 
 import numpy as np
 
-from tscode.fast_algebra import norm, norm_of
+from tscode.algebra import norm, norm_of, rot_mat_from_pointer, vec_angle
+from tscode.graph_manipulations import neighbors
 from tscode.parameters import orb_dim_dict
-from tscode.utils import neighbors, pt, rot_mat_from_pointer, vec_angle
+from tscode.pt import pt
 
 
 class Single:
@@ -35,20 +36,20 @@ class Single:
         '''
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
 
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.other = mol.atomcoords[conf][neighbors_indexes][0]
+        self.other = mol.atomcoords[conf][neighbors_indices][0]
 
         if not mol.sp3_sigmastar:
             self.orb_vecs = np.array([norm(self.coord - self.other)])
 
         else:
-            other_reactive_indexes = list(mol.reactive_indexes)
-            other_reactive_indexes.remove(i)
-            for index in other_reactive_indexes:
-                    if index in neighbors_indexes:
+            other_reactive_indices = list(mol.reactive_indices)
+            other_reactive_indices.remove(i)
+            for index in other_reactive_indices:
+                    if index in neighbors_indices:
                         parnter_index = index
                         break
             # obtain the reference partner index
@@ -69,7 +70,7 @@ class Single:
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
 
                 if orb_dim is None:
@@ -89,13 +90,13 @@ class Sp2:
         '''
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
         
 
 
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
 
         self.vectors = self.others - self.coord # vectors connecting reactive atom with neighbors
         self.orb_vec = norm(np.mean(np.array([np.cross(norm(self.vectors[0]), norm(self.vectors[1])),
@@ -106,7 +107,7 @@ class Sp2:
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
                 
                 if orb_dim is None:
@@ -127,10 +128,10 @@ class Sp3:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        neighbors_indices = neighbors(mol.graph, i)
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
         
         if not mol.sp3_sigmastar:
 
@@ -145,7 +146,7 @@ class Sp3:
 
             else: # if we cannot infer, ask user if we didn't have already 
                 try:
-                    self.leaving_group_coords = self._set_leaving_group(mol, neighbors_indexes)
+                    self.leaving_group_coords = self._set_leaving_group(mol, neighbors_indices)
 
                 except Exception:
                 # if something goes wrong, we fallback to command line input for reactive atom index collection
@@ -159,31 +160,31 @@ class Sp3:
                             if self.leaving_group_index == '' or self.leaving_group_index.lower().islower():
                                 pass
                             
-                            elif int(self.leaving_group_index) in neighbors_indexes:
+                            elif int(self.leaving_group_index) in neighbors_indices:
                                 self.leaving_group_index = int(self.leaving_group_index)
                                 break
 
                             else:
                                 print(f'Atom {self.leaving_group_index} is not bonded to the sp3 center with index {self.index}.')
                     
-                    self.leaving_group_coords = self.others[neighbors_indexes.index(self.leaving_group_index)]
+                    self.leaving_group_coords = self.others[neighbors_indices.index(self.leaving_group_index)]
 
             self.orb_vecs = np.array([self.coord - self.leaving_group_coords])
             self.orb_vers = norm(self.orb_vecs[0])
 
         else: # Sigma bond type
 
-            other_reactive_indexes = list(mol.reactive_indexes)
-            other_reactive_indexes.remove(i)
-            for index in other_reactive_indexes:
-                    if index in neighbors_indexes:
+            other_reactive_indices = list(mol.reactive_indices)
+            other_reactive_indices.remove(i)
+            for index in other_reactive_indices:
+                    if index in neighbors_indices:
                         parnter_index = index
                         break
             # obtain the reference partner index
 
             pivot = norm(mol.atomcoords[conf][parnter_index] - self.coord)
 
-            other_neighbors = deepcopy(neighbors_indexes)
+            other_neighbors = deepcopy(neighbors_indices)
             other_neighbors.remove(parnter_index)
             orb_vec = norm(mol.atomcoords[conf][other_neighbors[0]] - self.coord)
             orb_vec = orb_vec - orb_vec @ pivot * pivot
@@ -196,7 +197,7 @@ class Sp3:
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
                 
                 if orb_dim is None:
@@ -205,7 +206,7 @@ class Sp3:
 
             self.center = np.array([orb_dim * norm(vec) + self.coord for vec in self.orb_vecs])
 
-    def _set_leaving_group(self, mol, neighbors_indexes):
+    def _set_leaving_group(self, mol, neighbors_indices):
         '''
         Manually set the molecule leaving group from the ASE GUI, imposing
         a constraint on the desired atom.
@@ -230,7 +231,7 @@ class Sp3:
                 
                 if atoms.constraints != []:
                     if len(list(atoms.constraints[0].get_indices())) == 1:
-                        if list(atoms.constraints[0].get_indices())[0] in neighbors_indexes:
+                        if list(atoms.constraints[0].get_indices())[0] in neighbors_indices:
                             self.leaving_group_index = list(atoms.constraints[0].get_indices())[0]
                             break
                         else:
@@ -241,7 +242,7 @@ class Sp3:
                         atoms.constraints = []
 
 
-        return self.others[neighbors_indexes.index(self.leaving_group_index)]
+        return self.others[neighbors_indices.index(self.leaving_group_index)]
 
 
 class Ether:
@@ -254,19 +255,19 @@ class Ether:
         '''
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
         
 
 
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
 
         self.orb_vecs = self.others - self.coord # vectors connecting center to each of the two substituents
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
                 
                 if orb_dim is None:
@@ -275,7 +276,7 @@ class Ether:
 
             self.orb_vecs = orb_dim * np.array([norm(v) for v in self.orb_vecs]) # making both vectors a fixed, defined length
 
-            orb_mat = rot_mat_from_pointer(np.mean(self.orb_vecs, axis=0), 90) @ rot_mat_from_pointer(norm(np.cross(self.orb_vecs[0], self.orb_vecs[1])), 180)
+            orb_mat = rot_mat_from_pointer(np.mean(self.orb_vecs, axis=0), 90) @ rot_mat_from_pointer(np.cross(self.orb_vecs[0], self.orb_vecs[1]), 180)
 
             # self.orb_vecs = np.array([orb_mat @ v for v in self.orb_vecs])
             self.orb_vecs = (orb_mat @ self.orb_vecs.T).T
@@ -287,60 +288,45 @@ class Ether:
 class Ketone:
     
     def __repr__(self):
-        return 'Ketone'
+        return f'Ketone ({self.subtype})'
 
     def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)       
+        neighbors_indices = neighbors(mol.graph, i)       
+        self.subtype = 'pre-init'
 
 
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.other = mol.atomcoords[conf][neighbors_indexes][0]
+        self.other = mol.atomcoords[conf][neighbors_indices][0]
 
         self.vector = self.other - self.coord # vector connecting center to substituent
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
                 
                 if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
 
-            neighbors_of_neighbor_indexes = neighbors(mol.graph, neighbors_indexes[0])
-            neighbors_of_neighbor_indexes.remove(i)
+            neighbors_of_neighbor_indices = neighbors(mol.graph, neighbors_indices[0])
+            neighbors_of_neighbor_indices.remove(i)
 
             self.vector = norm(self.vector)*orb_dim
 
-            if len(neighbors_of_neighbor_indexes) == 2:
-            # if it is a normal ketone (or an enolate), n orbital lobes must be coplanar with
-            # atoms connecting to ketone C atom, or p lobes must be placed accordingly
-
-                a1 = mol.atomcoords[conf][neighbors_of_neighbor_indexes[0]]
-                a2 = mol.atomcoords[conf][neighbors_of_neighbor_indexes[1]]
-                pivot = norm(np.cross(a1 - self.coord, a2 - self.coord))
-
-                if mol.sigmatropic[conf]:
-                    # two p lobes
-                    self.center = np.concatenate(([pivot*orb_dim], [-pivot*orb_dim]))
-
-                else:
-                    #two n lobes
-                    self.center = np.array([rot_mat_from_pointer(pivot, angle) @ self.vector for angle in (120,240)])
-
-            elif len(neighbors_of_neighbor_indexes) in (1, 3):
-            # ketene or alkoxide
+            if len(neighbors_of_neighbor_indices) == 1:
+            # ketene
             
-                ketene_sub_indexes = neighbors(mol.graph, neighbors_of_neighbor_indexes[0])
-                ketene_sub_indexes.remove(neighbors_indexes[0])
+                ketene_sub_indices = neighbors(mol.graph, neighbors_of_neighbor_indices[0])
+                ketene_sub_indices.remove(neighbors_indices[0])
 
-                ketene_sub_coords = mol.atomcoords[conf][ketene_sub_indexes[0]]
-                n_o_n_coords = mol.atomcoords[conf][neighbors_of_neighbor_indexes[0]]
+                ketene_sub_coords = mol.atomcoords[conf][ketene_sub_indices[0]]
+                n_o_n_coords = mol.atomcoords[conf][neighbors_of_neighbor_indices[0]]
 
                 # vector connecting ketene R with C (O=C=C(R)R)
                 v = (ketene_sub_coords - n_o_n_coords)
@@ -350,7 +336,37 @@ class Ketone:
                 pointer = norm(pointer) * orb_dim
 
                 self.center = np.array([rot_mat_from_pointer(self.vector, 90*step) @ pointer for step in range(4)])
+
+                self.subtype = 'p+p'
             
+            elif len(neighbors_of_neighbor_indices) == 2:
+            # if it is a normal ketone (or an enolate), n orbital lobes must be coplanar with
+            # atoms connecting to ketone C atom, or p lobes must be placed accordingly
+
+                a1 = mol.atomcoords[conf][neighbors_of_neighbor_indices[0]]
+                a2 = mol.atomcoords[conf][neighbors_of_neighbor_indices[1]]
+                pivot = norm(np.cross(a1 - self.coord, a2 - self.coord))
+
+                if mol.sigmatropic[conf]:
+                    # two p lobes
+                    self.center = np.concatenate(([pivot*orb_dim], [-pivot*orb_dim]))
+                    self.subtype = 'p'
+
+                else:
+                    #two n lobes
+                    self.center = np.array([rot_mat_from_pointer(pivot, angle) @ self.vector for angle in (120,240)])
+                    self.subtype = 'sp2'
+
+            elif len(neighbors_of_neighbor_indices) == 3:
+            # alkoxide, sulfonamide
+
+                v1, v2, v3 = mol.atomcoords[conf][neighbors_of_neighbor_indices] - self.coord
+                v1, v2, v3 = norm(v1), norm(v2), norm(v3)
+                v1, v2, v3 = v1*orb_dim, v2*orb_dim, v3*orb_dim
+                pivot = norm(np.cross(self.vector, v1))
+
+                self.center = np.array([rot_mat_from_pointer(pivot, 180) @ v for v in (v1, v2, v3)])
+                self.subtype = 'trilobe'
         
             self.orb_vecs = np.array([norm(center) for center in self.center])
             # unit vectors connecting reactive atom coord with orbital centers
@@ -369,19 +385,19 @@ class Imine:
         '''
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
         
 
 
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
 
         self.vectors = self.others - self.coord # vector connecting center to substituent
 
         if update:
             if orb_dim is None:
-                key = self.symbol + ' ' + str(self)
+                key = self.symbol + ' ' + str(self).split(' (')[0]
                 orb_dim = orb_dim_dict.get(key)
                 
                 if orb_dim is None:
@@ -410,12 +426,12 @@ class Sp_or_carbene:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
         
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
 
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
 
         self.vectors = self.others - self.coord # vector connecting center to substituent
 
@@ -433,33 +449,33 @@ class Sp_or_carbene:
         self.ketene = False
         if self.type == 'sp' and all([s == 'C' for s in self.neighbors_symbols]):
 
-            neighbors_of_neighbors_indexes = (neighbors(mol.graph, neighbors_indexes[0]),
-                                              neighbors(mol.graph, neighbors_indexes[1]))
+            neighbors_of_neighbors_indices = (neighbors(mol.graph, neighbors_indices[0]),
+                                              neighbors(mol.graph, neighbors_indices[1]))
 
-            neighbors_of_neighbors_indexes[0].remove(i)
-            neighbors_of_neighbors_indexes[1].remove(i)
+            neighbors_of_neighbors_indices[0].remove(i)
+            neighbors_of_neighbors_indices[1].remove(i)
 
-            if (len(side1) == len(side2) == 2 for side1, side2 in neighbors_of_neighbors_indexes):
+            if (len(side1) == len(side2) == 2 for side1, side2 in neighbors_of_neighbors_indices):
                 self.allene = True
 
         elif self.type == 'sp' and sorted(self.neighbors_symbols) in (['C', 'O'], ['C', 'S']):
 
             self.ketene = True
 
-            neighbors_of_neighbors_indexes = (neighbors(mol.graph, neighbors_indexes[0]),
-                                              neighbors(mol.graph, neighbors_indexes[1]))
+            neighbors_of_neighbors_indices = (neighbors(mol.graph, neighbors_indices[0]),
+                                              neighbors(mol.graph, neighbors_indices[1]))
 
-            neighbors_of_neighbors_indexes[0].remove(i)
-            neighbors_of_neighbors_indexes[1].remove(i)
+            neighbors_of_neighbors_indices[0].remove(i)
+            neighbors_of_neighbors_indices[1].remove(i)
                 
-            if len(neighbors_of_neighbors_indexes[0]) == 2:
-                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indexes[0][0]]
-                ketene_atom = mol.atomcoords[conf][neighbors_indexes[0]]
+            if len(neighbors_of_neighbors_indices[0]) == 2:
+                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indices[0][0]]
+                ketene_atom = mol.atomcoords[conf][neighbors_indices[0]]
                 self.ketene_ref = substituent - ketene_atom
 
-            elif len(neighbors_of_neighbors_indexes[1]) == 2:
-                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indexes[1][0]]
-                ketene_atom = mol.atomcoords[conf][neighbors_indexes[1]]
+            elif len(neighbors_of_neighbors_indices[1]) == 2:
+                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indices[1][0]]
+                ketene_atom = mol.atomcoords[conf][neighbors_indices[1]]
                 self.ketene_ref = substituent - ketene_atom
 
             else:
@@ -488,8 +504,8 @@ class Sp_or_carbene:
                     # versor connecting reactive atom neighbors
                     
                     if self.allene:
-                        ref = (mol.atomcoords[conf][neighbors_of_neighbors_indexes[0][0]] -
-                               mol.atomcoords[conf][neighbors_indexes[0]])
+                        ref = (mol.atomcoords[conf][neighbors_of_neighbors_indices[0][0]] -
+                               mol.atomcoords[conf][neighbors_indices[0]])
                     else:
                         ref = self.ketene_ref
 
@@ -531,18 +547,18 @@ class Metal:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
-        neighbors_indexes = neighbors(mol.graph, i)
+        neighbors_indices = neighbors(mol.graph, i)
         
-        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
+        self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indices]
         self.coord = mol.atomcoords[conf][i]
-        self.others = mol.atomcoords[conf][neighbors_indexes]
+        self.others = mol.atomcoords[conf][neighbors_indices]
 
         self.vectors = self.others - self.coord # vectors connecting reactive atom with neighbors
 
         v1 = self.vectors[0]
         # v1 connects first bonded atom to the metal itself
 
-        neighbor_of_neighbor_index = neighbors(mol.graph, neighbors_indexes[0])[0]
+        neighbor_of_neighbor_index = neighbors(mol.graph, neighbors_indices[0])[0]
         v2 = mol.atomcoords[conf][neighbor_of_neighbor_index] - self.coord
         # v2 connects first neighbor of the first neighbor to the metal itself
 
@@ -561,38 +577,38 @@ class Metal:
 
 # Keys are made of atom symbol and number of bonds that it makes
 atom_type_dict = {
-            'H1' : Single(),
+            'H1' : Single,
 
-            'B3' : Sp2(),
-            'B4' : Sp3(),
+            'B3' : Sp2,
+            'B4' : Sp3,
 
-            'C1' : Single(), # deprotonated terminal alkyne. What if it is a carbylidene? Very rare by the way...
-            'C2' : Sp_or_carbene(), # sp if straight, carbene if bent
-            'C3' : Sp2(), # double ball
-            'C4' : Sp3(), # one ball, on the back of the leaving group. If we can't tell which one it is, we ask user
+            'C1' : Single, # deprotonated terminal alkyne. What if it is a carbylidene? Very rare by the way...
+            'C2' : Sp_or_carbene, # sp if straight, carbene if bent
+            'C3' : Sp2, # double ball
+            'C4' : Sp3, # one ball, on the back of the leaving group. If we can't tell which one it is, we ask user
 
-            'N1' : Single(),
-            'N2' : Imine(), # one ball on free side
-            'N3' : Sp2(), # double ball
-            'N4' : Sp3(), # leaving group
+            'N1' : Single,
+            'N2' : Imine, # one ball on free side
+            'N3' : Sp2, # double ball
+            'N4' : Sp3, # leaving group
 
-            'O1' : Ketone(), # two balls 120° apart. Also for alkoxides, good enough
-            'O2' : Ether(), # or alcohol, two balls about 109,5° apart
+            'O1' : Ketone, # two balls 120° apart. Also for alkoxides, good enough
+            'O2' : Ether, # or alcohol, two balls about 109,5° apart
 
-            'P2' : Imine(), # one ball on free side
-            'P3' : Sp2(), # double ball
-            'P4' : Sp3(), # leaving group
+            'P2' : Imine, # one ball on free side
+            'P3' : Sp2, # double ball
+            'P4' : Sp3, # leaving group
 
-            'S1' : Ketone(),
-            'S2' : Ether(),
-            'S3' : Sp2(), # Not sure if this can be valid, but it's basically treating it as a bent carbonyl, should work
-            #  'S3' : Sulphoxide(), # Should we consider this? Or just ok with Sp2()?
-            # 'S4' : Sulphone(),
+            'S1' : Ketone,
+            'S2' : Ether,
+            'S3' : Sp2, # Not sure if this can be valid, but it's basically treating it as a bent carbonyl, should work
+            #  'S3' : Sulphoxide, # Should we consider this? Or just ok with Sp2()?
+            # 'S4' : Sulphone,
 
-            'F1' : Single(),
-            'Cl1': Single(),
-            'Br1': Single(),
-            'I1' : Single(),
+            'F1' : Single,
+            'Cl1': Single,
+            'Br1': Single,
+            'I1' : Single,
              }
 
 metals = (
@@ -612,4 +628,12 @@ metals = (
 for metal in metals:
     for bonds in range(1,9):
         bonds = str(bonds)
-        atom_type_dict[metal+bonds] = Metal()
+        atom_type_dict[metal+bonds] = Metal
+
+def get_atom_type(graph, index):
+    '''
+    Returns the appropriate class to represent
+    the atom with the given index on the graph
+    '''
+    nb = neighbors(graph, index)
+    return atom_type_dict[pt[graph.nodes[index]['atomnos']].symbol + str(len(nb))]
\ No newline at end of file
