@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -59,14 +59,17 @@ DEFAULT_LEVELS = {
 
 COMMANDS = {
     'MOPAC':'MOPAC2016.exe',
-    'ORCA':'orca.exe',
+    'ORCA':'/vast/palmer/apps/avx.grace/software/ORCA/5.0.4-gompi-2020b/bin/orca',
     'GAUSSIAN':'g09.exe',
+    'XTB':'xtb',
 }
 # Command with which calculators will be called from the command line
 
-PROCS = 8
-# Number of processors (cores) to be used by ORCA and/or Gaussian
+PROCS = 0
+# Number of processors (cores) to be used by XTB, ORCA and/or Gaussian (0 is auto)
 
-MEM_GB = 1
-# Memory allocated for each job (Gaussian only). If you experience problems
-# in running Gaussian calculation, try setting PROCS to 1 and MEM_GB to 0.5.
\ No newline at end of file
+THREADS = 0
+# Number of threads for concurrent geometry optimizations (0 is auto)
+
+MEM_GB = 8
+# Memory allocated for each job (Gaussian/ORCA)
\ No newline at end of file
