@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2022 Nicolò Tampellini
+Copyright (C) 2021-2023 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@ GNU General Public License for more details.
 
 '''
 
+import sys
+
 xtb_solvents = [
     'acetone',
     'acetonitrile',
@@ -90,7 +92,7 @@ solvent_synonyms = {
     'etoh':'ethanol',
     'ch3oh':'methanol',
     'meoh':'methanol',
-    'h2o':'water',    
+    'h2o':'water',
 }
 
 new_theory_level = {
@@ -113,6 +115,6 @@ def get_solvent_line(solvent, calculator, theory_level):
         for s in epsilon_dict:
             print('    '+s)
         print('Please note that not all solvents will work with all calculators.')
-        quit()
+        sys.exit()
 
     return new_theory_level[calculator](theory_level, solvent)
\ No newline at end of file
